#include <avr/io.h>
#include <avr/interrupt.h>

#include "HX711.h"
#include "macros.h"
#include "pins.h"


//======================================================================================================================
// Module Level Variables
//----------------------------------------------------------------------------------------------------------------------
static uint8_t gain_m;       // amplification factor



//======================================================================================================================
//Private Functions
//----------------------------------------------------------------------------------------------------------------------
static uint8_t shiftIn(void)
{
	uint8_t value = 0;
	uint8_t i;

	for (i = 0; i < 8; ++i)
	{
		sbi(PORTB, PIN_CLK);
		value |= ((PINB >> PIN_DAT) & 0x01) << (7 - i);
		cbi(PORTB, PIN_CLK);
	}
	return value;
}


//======================================================================================================================
// Public Functions
//----------------------------------------------------------------------------------------------------------------------
void hx711_begin(uint8_t gain)
{
	sbi(DDRB, PIN_CLK);
	cbi(DDRB, PIN_DAT);

	hx711_set_gain(gain);
}

//----------------------------------------------------------------------------------------------------------------------
uint8_t hx711_is_ready(void)
{
	return (PINB & (1 << PIN_DAT)) == 0;
}

//----------------------------------------------------------------------------------------------------------------------
void hx711_set_gain(uint8_t gain)
{
	switch (gain)
	{
		case 128:		// channel A, gain factor 128
			gain_m = 1;
			break;
		case 64:		// channel A, gain factor 64
			gain_m = 3;
			break;
		case 32:		// channel B, gain factor 32
			gain_m = 2;
			break;
	}

	cbi(PORTB, PIN_CLK);
	hx711_read();
}

//----------------------------------------------------------------------------------------------------------------------
uint32_t hx711_read(void)
{
	// wait for the chip to become ready
	while (!hx711_is_ready())
	{
		// Do Nothing.
	}

	uint32_t value = 0;
	uint8_t data[3] = { 0 };
	uint8_t filler = 0x00;

	cli();

	// pulse the clock pin 24 times to read the data
	data[2] = shiftIn();
	data[1] = shiftIn();
	data[0] = shiftIn();

	// set the channel and the gain factor for the next reading using the clock pin
	for (uint8_t i = 0; i < gain_m; i++)
	{
		sbi(PORTB, PIN_CLK);
		cbi(PORTB, PIN_CLK);
	}

	sei();

	// Replicate the most significant bit to pad out a 32-bit signed integer
	if (data[2] & 0x80)
	{
		filler = 0xFF;
	}
	else
	{
		filler = 0x00;
	}

	// Construct a 32-bit signed integer
	value = ( ((uint32_t)filler) << 24
			| ((uint32_t)data[2]) << 16
			| ((uint32_t)data[1]) << 8
			| ((uint32_t)data[0]) );

	return value;
}

//----------------------------------------------------------------------------------------------------------------------
void hx711_power_down(void)
{
	cbi(PORTB, PIN_CLK);
	sbi(PORTB, PIN_CLK);
}

//----------------------------------------------------------------------------------------------------------------------
void hx711_power_up(void)
{
	cbi(PORTB, PIN_CLK);
}

/* End of File */