/*======================================================================================================================
 *      File: main.c
 *   Project: Parts Scale
 * Processor: ATtiny85
 * Frequency: 16.5 Mhz
 *    Author: eX Systems
 *     Email: jaredjulien@exsystems.net
 * Copyright: (c) 2018 eX Systems
 *
 * --------------------------------------------------------------------------------------------------------------------
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
 * OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * --------------------------------------------------------------------------------------------------------------------
 *
 * Date        Rev   Author    Change Description
 * ==========  ====  ========  ========================================================================================
 * 2018-02-11  1     Jared     Initial file creation based upon USB ignition switch project
 * 2018-02-11  2     Jared     Reduced command set to be more of a direct interface to HX711
 * 2018-02-15  3     Jared     Added support for DS18B20 temperature sensor
 * 2018-02-20  4     Jared     Removed support for the "millis" counter ISR
 * -------------------------------------------------------------------------------------------------------------------*/
#define VERSION 3

//======================================================================================================================
// Header Files
//----------------------------------------------------------------------------------------------------------------------
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/power.h>
#include <avr/wdt.h>
#include <util/delay.h>

#include "usbconfig.h"
#include "usbdrv/usbdrv.h"
#include "hx711.h"
#include "ds18b20.h"
#include "macros.h"
#include "pins.h"


//======================================================================================================================
// USB Commands
//----------------------------------------------------------------------------------------------------------------------
#define CMD_GET_VERSION  0x00
#define CMD_SET_GAIN     0x01
#define CMD_PWR_OFF      0x10
#define CMD_PWR_ON       0x11
#define CMD_READ         0x20
#define CMD_TEMPERATURE  0x30


//======================================================================================================================
// Module Level Variables
//----------------------------------------------------------------------------------------------------------------------
static uint8_t replyBuf[4U];
static int16_t temperature;


//======================================================================================================================
// USB Functions
//----------------------------------------------------------------------------------------------------------------------
usbMsgLen_t usbFunctionSetup(uint8_t data[8U])
{
	usbRequest_t *rq = (void *)data;
	usbMsgPtr = (unsigned short)replyBuf;
	if(rq->bRequest == CMD_GET_VERSION)
	{
		replyBuf[0] = VERSION >> 8;
		replyBuf[1] = VERSION & 0xFF;
		return 2U;
	}
	else if (rq->bRequest == CMD_SET_GAIN)
	{
		hx711_set_gain(rq->wValue.bytes[0U]);
	}
	else if (rq->bRequest == CMD_PWR_OFF)
	{
		hx711_power_down();
	}
	else if (rq->bRequest == CMD_PWR_ON)
	{
		hx711_power_up();
	}
	else if (rq->bRequest == CMD_READ)
	{
		uint32_t measured = hx711_read();
		replyBuf[0] = (measured >> 16) & 0xFF;
		replyBuf[1] = (measured >> 8) & 0xFF;
		replyBuf[2] = measured & 0xFF;
		return 3U;
	}
	else if (rq->bRequest == CMD_TEMPERATURE)
	{
		replyBuf[0] = temperature >> 8;
		replyBuf[1] = temperature & 0xFF;
		return 2;
	}
	return 0;
}


//======================================================================================================================
// Main Function
//----------------------------------------------------------------------------------------------------------------------
int main(void)
{
	uint8_t i;
	uint8_t error;

	cli();
	wdt_enable(WDTO_1S);

	// Setup load cell amplifier.
	hx711_begin(128);

	// Setup the temperature sensor.
	ds18b20_init();

	// run at full speed, because Trinket defaults to 8MHz for low voltage compatibility reasons
	clock_prescale_set(clock_div_1);

	// fake a disconnect to force the computer to re-enumerate
	PORTB &= ~(_BV(USB_CFG_DMINUS_BIT) | _BV(USB_CFG_DPLUS_BIT));
	usbDeviceDisconnect();
	for (i = 0U; i < 250U; i++) // wait 500 ms
	{
		wdt_reset(); // keep the watchdog happy
		_delay_ms(2);
	}
	usbDeviceConnect();

	// start the USB driver
	usbInit();
	sei();

	while (1)
	{
		wdt_reset();
		usbPoll();

		error = ds18b20_getTemp(&temperature);
		if (error == DS18B20_ERR_OK)
		{
			ds18b20_startConversion();
		}
	}
}

/* End of File: main.c */
